// $Id: BitMask.h,v 1.1.2.2 1998/11/18 18:49:41 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef BIT_MASK_CLASS_H
#define BIT_MASK_CLASS_H

// fixed size Bit mask
class BitMask {
	protected:
		typedef int Bin;

	protected:
		static int BinOff(int idx) { return idx / sizeof(Bin); }
		static int BitOff(int idx) { return idx % sizeof(Bin); }
		static int BitVal(int idx) { return 1 << BitOff(idx); }
		static int BinCount(int bitCount) { return BinOff(bitCount) + 1; }

	public:
		BitMask(int aCapacity = 0): theBins(0), theCapacity(0), theCount(0) { resize(aCapacity); }
		~BitMask() { delete[] theBins; }

		int capacity() const { return theCapacity; }
		int count() const { return theCount; }

		bool isSet(int bit) const { return (bin(bit) & BitVal(bit)) != 0; }
		bool operator [](int bit) const { return isSet(bit); }

		void beSet(int bit) { bin(bit) |= BitVal(bit); theCount++; }
		void beClear(int bit) { bin(bit) &= ~BitVal(bit); theCount--; }

		void resize(int aCapacity);
		void clear();

	protected:
		Bin &bin(int bit) { return theBins[BinOff(bit)]; }
		const Bin &bin(int bit) const { return theBins[BinOff(bit)]; }

	protected:
		Bin *theBins;
		int theCapacity;  // in bits
		int theCount;     // not a number of bits set! see beSet/beClear
};


#endif
