// $Id

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef _KERNEL_CLASS_H_
#define _KERNEL_CLASS_H_

#include "AppDef.h"
#include "Array.h"

class Process;
class Thread;

// scheduler for threads
class Kernel {
	public:
		Kernel();
		~Kernel();

		bool idle() const { return theActivists.count() <= 0; }

		void slice();                // give all active threads a slice of CPU

		// start execution of a new thread, returns kidx
		int start(Thread *th);
		// stop execution of a thread and destroy it
		void stop(Thread *th) { if (th) remove(th); }
		// make existing thread (usually "this") active
		void activate(Thread *th);

		// parents can use these
		void stopKid(Process *parent, int idx); // stop execution of a kid if found
		void stopKids(Process *parent);  // stop execution of all parent's kids if any

	protected:
		void remove(Thread *th);

	protected:
		SchArray<Thread*> theThreads;   // all threads
		SchArray<Thread*> theActivists; // active threads
		int theActOffset;               // first active not-processed thread
		int theCcLevel;                 // number of alive threads
		int theCcLevelToReport;         // threshold for next growth report
};

extern Kernel TheKernel;

#endif
