#ifndef _LOG_CLASSES_H_
#define _LOG_CLASSES_H_ "$Id: Log.h,v 1.8.2.2 1998/12/29 23:07:35 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"

#include <iostream.h>

// This module declares several interfaces that facilitate logging.
// Note: treat _all_ "names" here as macros.

// Logging levels:
//		Sys Log: administrative information for maintainers
//		Journal: essential information for other programs
//		Notepad: debugging information for programmers

// check if the logging is enabled and set appropriate #defines
#if DISABLE_LOGGING
#define LOG__ON false
#else
#define LOG__ON true
#endif

// kludge to get current source code location
#define theSrcLoc __FILE__ << ':' << __LINE__ << ":\t"

// every log entry starts with its context
#define SysLog  TheSysLog  << theSysCtx
#define Journal if (LOG__ON) TheJournal << theJorCtx
#define Notepad if (LOG__ON) TheNotepad << thePadCtx << theSrcLoc

// sometimes we need to append to an entry
#define SysLog_  TheSysLog
#define Journal_ if (LOG__ON) TheJournal
#define Notepad_ if (LOG__ON) TheNotepad

// every log entry must end with ende
inline ostream& ende(ostream &os) { return os << endl; }

// Report log is just a regular stream
#define Report TheReport

// Note: an object may easily extend the notion of "context" by
//       having appropriate "printable' members (e.g. theSysCtx).
//       The same is, of course, true for trailers

// Note: it is possible to implement logging so "ende" will know
//       the type of the log entry it terminates

extern ostream& theSysCtx(ostream& os);
extern ostream& theJorCtx(ostream& os);
extern ostream& thePadCtx(ostream& os);

extern ostream &TheSysLog;
extern ostream &TheNotepad;
extern ostream &TheJournal;
extern ostream &TheReport;

extern void ConfigureLogs(int precision, const char *fname = 0);

#endif
