#ifndef _PROGRESS_METER_H_
#define _PROGRESS_METER_H_ "$Id: ProgressMeter.h,v 1.4.2.2 1998/11/08 04:56:44 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "AppDef.h"
#include "globals.h"

class ostream;

// Progress Meter configuration
class ProgMeterCfg {
	public:
		ProgMeterCfg();

	public:
		int theMaxCcLevel;
		int theMaxStartCount;
};

// interface for stopping an iteration of something
class ProgressMeter {
	public:
		ProgressMeter(): theStartCount(0), theFinishCount(0) {}
		~ProgressMeter() {}

		void configure(const ProgMeterCfg &aCfg) { theCfg = aCfg; }
		ProgressMeter *clone() const;

		inline bool enough() const;
		int ccLevel() const { return theStartCount - theFinishCount; }

		void noteTaskStart() { theStartCount++; }
		void noteTaskFinish() { theFinishCount++; }
		void noteTaskFailed() { noteTaskStart(); noteTaskFinish(); }

		ostream &print(ostream &os) const;

	protected:
		ProgMeterCfg theCfg;

		int theStartCount;
		int theFinishCount;
};

inline 
bool ProgressMeter::enough() const { 
	return 
		(theCfg.theMaxStartCount >= 0 && theStartCount >= theCfg.theMaxStartCount) ||
//		(theCfg.theMaxCcLevel >= 0 && ccLevel() >= theCfg.theMaxCcLevel) ||
		TheWorld.goalReached();
}

#endif
