#ifndef _SERVER_H_
#define _SERVER_H_ "$Id: Server.h,v 1.18.2.2 1998/11/14 23:28:41 rousskov Exp $"

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "SelectLoop.h"
#include "Pause.h"
#include "ConnInfo.h"
#include "Xaction.h"
#include "TopProcess.h"
#include "AgingInfo.h"
#include "Array.h"

class ConnMgr;

// just to simplify configuration interface
class ServerCfg {
	public:
		ServerCfg();
		~ServerCfg() {} // does not destroy!

		ServerCfg clone() const;
		void destroy();

	public:
		ConnMgr *theConnMgr; /* not used */
		Pause *thePause;
		XactCfg theXactCfg;
};

// emulates a Web server
class Server: public TopProcess, public IOer, public ResOwner {
	public:
		Server();
		virtual ~Server();

		virtual Thread *clone() const;

		void configure(const ServerCfg &aCfg);
		void ports(const IntArray &ports);
		void port(int port);

		virtual void init() { state(thSleeping); }
		virtual void run() {}

		virtual void idleRes(ResMgr *mgr);
		virtual void voidRes(ResMgr *mgr);
		virtual void readReady(int fd); 

		void checkPorts();

		virtual const char *name() const { return "Server"; }

	protected:
		virtual void beIdle() {} // it's OK for a server to be idle

		void listen(int port);
		bool accept(int fd);
		void deaf(int idx);
		void launchXact(ConnInfo &ci);

		bool find(ConnInfo *ci, int &idx) const;
		bool existConn(ResMgr *mgr) const;
		void closeConn(int idx);

	protected:
		ServerCfg theCfg;
		SchArray<Socket> theListenSockets;
		SchArray<ConnInfo*> theConns;

		int theNextPortIdx;  // see checkPorts()
};

#endif
