// $Id: Xaction.h,v 1.17.2.5 1998/12/15 07:28:06 rousskov Exp $

/*
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation, and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef _XACTION_H_
#define _XACTION_H_

#include "Process.h"
#include "XactIOThread.h"
#include "AlarmClock.h"
#include "ContThread.h"

class XactRec;

// just to simplify configuration interface
class XactCfg {
	public:
		XactCfg();
		~XactCfg() {} // does not destroy members

		XactCfg clone() const;
		void destroy();

	public:
		ContThread* theContMaker;
		ContThread* theContEater;
		XactIOThread* theReader;
		XactIOThread* theWriter;
};

// transaction is a (make, write, read, eat) set of
// possibly concurrent activities
class Xaction: public Process, public Sleeper {
	public:
		Xaction() {}
		virtual ~Xaction();

		virtual Thread *clone() const;

		virtual void configure(const XactCfg &aCfg);
		virtual void conn(const ConnMx &inMx, const ConnMx &outMx);

		virtual const char *name() const { return "Xaction"; }

		virtual void init() { theParent->taskStarted(this); beActive(thRunnable); }
		virtual void run();

		virtual void wakeUp(const Alarm &alarm);

	protected:
		void startPushPull(XactThread *pusher, XactThread *puller, ConnMx &conn);

	protected:
		XactCfg theCfg;
		ConnMx theInMx;
		ConnMx theOutMx;

	public:
		// XXX: temporary place to store per-xaction measurements
		Time theStart;
		Size theRespSize;
		Size theReqSize;
};

#endif
